==> shelf_bind-0.5.0.tar.gz: lib/src/binding/handler_bindings.dart <==
// Copyright (c) 2014, The Shelf Bind project authors.
// Please see the AUTHORS file for details.
// All rights reserved. Use of this source code is governed by
// a BSD 2-Clause License that can be found in the LICENSE file.

library shelf_bind.bindings.handler;

import 'package:shelf/shelf.dart' show Handler, Request, Response;
import 'dart:mirrors' show ClosureMirror, reflect;
import '../preconditions.dart' show ensure, isNotNull;
import '../util.dart' show syncFuture;
import 'dart:async' show Future;
import 'package:path/path.dart' as p show url;

import 'type_bindings.dart' show BindMode, TypeBinding;
import 'parameter_bindings.dart' show HandlerParameterBinding,
    TypeBodyParameterBinding, TypePathParameterBinding,
    inferHandlerParameterBindings, CustomObjectFactory;
import 'response_bindings.dart' show JsonResponseBinding,
    ResponseBinding, inferResponseBinding;

final _url = p.url;

HandlerBinding createHandlerBinding(Function handler,
    List<HandlerParameterBinding> providedBindings,
    ResponseBinding responseBinding,
    Map<Type, CustomObjectFactory> customObjects,
    bool validateParameters, bool validateReturn) {
  return _inferHandlerBindings(handler, providedBindings,
      responseBinding, customObjects, validateParameters, validateReturn);
}


@deprecated
HandlerParameterBinding createRequestParameterBinding(
    Type type, Symbol constructor, Map<String, Symbol> bindings,
    BindMode bindMode) {
  return new TypePathParameterBinding(
      new TypeBinding(type, constructorName: constructor,
          bindMode: bindMode, memberBindings: bindings),
          false);
}

@deprecated
HandlerParameterBinding createJsonBodyBinding(Type type) {
  return new TypeBodyParameterBinding(new TypeBinding(type),false);
}


class HandlerBinding {
  final List<HandlerParameterBinding> parameterBindings;
  final Function _boundHandler;
  final ResponseBinding responseBinding;

  HandlerBinding(this.parameterBindings, Function handler,
      [ ResponseBinding responseBinding ])
      : this._boundHandler = _wrapHandler(handler),
        this.responseBinding = responseBinding != null ?
            responseBinding : new JsonResponseBinding.ok() {
    ensure(parameterBindings, isNotNull);
    ensure(_boundHandler, isNotNull);
    parameterBindings.forEach((pb) => pb.validate());
  }


  Handler get handler => _handleRequest;

  dynamic _handleRequest(Request request) {
    final parameterFutureValues =
        parameterBindings.map((pb) => syncFuture(() =>
            pb.createInstance(request)));

    return Future.wait(parameterFutureValues)
      .then((parameterValues) {
        return _boundHandler(parameterValues, request, responseBinding);
      });

  }


}





typedef _BoundHandler(List<dynamic> positionalArguments, Request request,
    ResponseBinding responseBinding);

HandlerBinding _inferHandlerBindings(Function handler,
    List<HandlerParameterBinding> providedParameterBindings,
    ResponseBinding providedResponseBinding,
    Map<Type, CustomObjectFactory> customObjects,
    bool validateParameters, bool validateReturn) {
  final ClosureMirror functionMirror = reflect(handler);
  final inferredParameterBindings =
      inferHandlerParameterBindings(functionMirror, providedParameterBindings,
          customObjects, validateParameters);

  final inferredResponseBinding = inferResponseBinding(
      functionMirror, providedResponseBinding, validateReturn);

  return new HandlerBinding(inferredParameterBindings, handler,
      inferredResponseBinding);
}







_BoundHandler _wrapHandler(Function handler) {
  final ClosureMirror handlerMirror = reflect(handler);
  final functionMirror = handlerMirror.function;
  final params = functionMirror.parameters;
  final numParams = params.length;
  final returnType = functionMirror.returnType;

  return (List<dynamic> positionalArguments, Request request,
      ResponseBinding responseBinding) {
    final result = handlerMirror.apply(positionalArguments);

    final futureResponse = syncFuture(() => result.reflectee);

    return futureResponse.then((response) {
      if (response == null) {
        return new Response.internalServerError(body:
          'function ${functionMirror.simpleName} returned null');
      }
      return responseBinding.createResponse(request, reflect(response));
    });
  };
}

==> shelf_bind-0.6.0.tar.gz: lib/src/binding/handler_bindings.dart <==
// Copyright (c) 2014, The Shelf Bind project authors.
// Please see the AUTHORS file for details.
// All rights reserved. Use of this source code is governed by
// a BSD 2-Clause License that can be found in the LICENSE file.

library shelf_bind.bindings.handler;

import 'package:shelf/shelf.dart' show Handler, Request, Response;
import 'dart:mirrors' show ClosureMirror, reflect;
import '../preconditions.dart' show ensure, isNotNull;
import '../util.dart' show syncFuture;
import 'dart:async' show Future;
import 'package:path/path.dart' as p show url;

import 'type_bindings.dart' show BindMode, TypeBinding;
import 'parameter_bindings.dart' show HandlerParameterBinding,
    JsonBodyTypeParameterBinding, TypePathParameterBinding,
    inferHandlerParameterBindings, CustomObjectFactory;
import 'response_bindings.dart' show JsonResponseBinding,
    ResponseBinding, inferResponseBinding;

final _url = p.url;

HandlerBinding createHandlerBinding(Function handler,
    List<HandlerParameterBinding> providedBindings,
    ResponseBinding responseBinding,
    Map<Type, CustomObjectFactory> customObjects,
    bool validateParameters, bool validateReturn) {
  return _inferHandlerBindings(handler, providedBindings,
      responseBinding, customObjects, validateParameters, validateReturn);
}


@deprecated
HandlerParameterBinding createRequestParameterBinding(
    Type type, Symbol constructor, Map<String, Symbol> bindings,
    BindMode bindMode) {
  return new TypePathParameterBinding(
      new TypeBinding(type, constructorName: constructor,
          bindMode: bindMode, memberBindings: bindings),
          false);
}

@deprecated
HandlerParameterBinding createJsonBodyBinding(Type type) {
  return new JsonBodyTypeParameterBinding(new TypeBinding(type),false);
}


class HandlerBinding {
  final List<HandlerParameterBinding> parameterBindings;
  final Function _boundHandler;
  final ResponseBinding responseBinding;

  HandlerBinding(this.parameterBindings, Function handler,
      [ ResponseBinding responseBinding ])
      : this._boundHandler = _wrapHandler(handler),
        this.responseBinding = responseBinding != null ?
            responseBinding : new JsonResponseBinding.ok() {
    ensure(parameterBindings, isNotNull);
    ensure(_boundHandler, isNotNull);
    parameterBindings.forEach((pb) => pb.validate());
  }


  Handler get handler => _handleRequest;

  dynamic _handleRequest(Request request) {
    final parameterFutureValues =
        parameterBindings.map((pb) => syncFuture(() =>
            pb.createInstance(request)));

    return Future.wait(parameterFutureValues)
      .then((parameterValues) {
        return _boundHandler(parameterValues, request, responseBinding);
      });

  }


}





typedef _BoundHandler(List<dynamic> positionalArguments, Request request,
    ResponseBinding responseBinding);

HandlerBinding _inferHandlerBindings(Function handler,
    List<HandlerParameterBinding> providedParameterBindings,
    ResponseBinding providedResponseBinding,
    Map<Type, CustomObjectFactory> customObjects,
    bool validateParameters, bool validateReturn) {
  final ClosureMirror functionMirror = reflect(handler);
  final inferredParameterBindings =
      inferHandlerParameterBindings(functionMirror, providedParameterBindings,
          customObjects, validateParameters);

  final inferredResponseBinding = inferResponseBinding(
      functionMirror, providedResponseBinding, validateReturn);

  return new HandlerBinding(inferredParameterBindings, handler,
      inferredResponseBinding);
}







_BoundHandler _wrapHandler(Function handler) {
  final ClosureMirror handlerMirror = reflect(handler);
  final functionMirror = handlerMirror.function;
  final params = functionMirror.parameters;
  final numParams = params.length;
  final returnType = functionMirror.returnType;

  return (List<dynamic> positionalArguments, Request request,
      ResponseBinding responseBinding) {
    final result = handlerMirror.apply(positionalArguments);

    final futureResponse = syncFuture(() => result.reflectee);

    return futureResponse.then((response) {
      if (response == null) {
        return new Response.internalServerError(body:
          'function ${functionMirror.simpleName} returned null');
      }
      return responseBinding.createResponse(request, reflect(response));
    });
  };
}
